/*
******************************************************************************
**
** @file        : LinkerScript.ld
**
** @author      : Auto-generated by STM32CubeIDE
**
**  Abstract    : Linker script for NUCLEO-F303K8 Board embedding STM32F303K8Tx Device from stm32f3 series
**                      64KBytes FLASH
**                      4KBytes CCMRAM
**                      12KBytes RAM
**
**                Set heap size, stack size and stack location according
**                to application requirements.
**
**                Set memory bank area and size if external memory is used
**
**  Target      : STMicroelectronics STM32
**
**  Distribution: The file is distributed as is, without any warranty
**                of any kind.
**
******************************************************************************
** @attention
**
** Copyright (c) 2023 STMicroelectronics.
** All rights reserved.
**
** This software is licensed under terms that can be found in the LICENSE file
** in the root directory of this software component.
** If no LICENSE file comes with this software, it is provided AS-IS.
**
******************************************************************************
*/

/* Entry Point */
ENTRY(Reset_Handler)

/* Highest address of the user mode stack */
/*_estack = ORIGIN(RAM) + LENGTH(RAM);*/ /* end of "RAM" Ram type memory */

/* Stack ins 4K große CCM-RAM verlegt. Das "Core-Coupled-Memory" ist im Zugriff schneller.
   Sollten aber die 4K nicht ausreichen, muss der Stack wieder ins normale RAM zurückverlegt werden.
*/

_estack = ORIGIN(CCMRAM) + LENGTH(CCMRAM); /* end of "CCM-RAM" Ram type memory */


_Min_Heap_Size = 0x200; /* required amount of heap */
_Min_Stack_Size = 0x400; /* required amount of stack */


/* Memory layout: 

            0                                                               FLASH_SIZE
            :                                                                   :            
            +----------------+----------------------------+-----------------+---+
     Flash: | PGMLDR_SECTION |        APPL_SECTION        | STORING_SECTION  CAS|
            +----------------+----------------------------+-----------------+---+
            |                |                            |                 | 
            +-------------+  |                            |                 |
                          |  |                            |                 |
  PGMLDR_SECTION_START ---+  |                            |                 |
  APPL_SECTION_START --------+                            |                 |
  STORING_SECTION_START ----------------------------------+                 |
  CY4ADR_SECTION_START -----------------------------------------------------+     

  - Sektion CAS (CY4ADR_SECTION) ist nur wenige Bytes groß und integraler Bestandteil
    der STORING_SECTION.
    
*/

FLASH_SIZE          = 64K;
PGMLDR_SIZE         = 4K;		/* Vielfaches Seitengröße */
STORING_SIZE        = 4K;		/* Vielfaches Seitengrenzen */
CY4ADR_SIZE         = 8;   	    /* Bytes ! */
FLASH_PAGE_BIT_SIZE = 11;       /* Seitengröße in Bits */

MEMORY
{    		
    RAM             (xrw) : ORIGIN = 0x20000000, LENGTH = 12K
	CCMRAM          (xrw) : ORIGIN = 0x10000000, LENGTH = 4K
	
    PGMLDR_SECTION  (rx)  : ORIGIN = 0x08000000, LENGTH = PGMLDR_SIZE
    APPL_SECTION    (rx)  : ORIGIN = 0x08000000 + PGMLDR_SIZE, LENGTH = FLASH_SIZE - PGMLDR_SIZE - STORING_SIZE	
	STORING_SECTION (rx)  : ORIGIN = 0x08000000 + FLASH_SIZE - STORING_SIZE, LENGTH = STORING_SIZE
	CY4ADR_SECTION  (rx)  : ORIGIN = 0x08000000 + FLASH_SIZE - CY4ADR_SIZE, LENGTH = CY4ADR_SIZE
}


/* Sections */
SECTIONS
{	
  /* The startup code into "APPL_SECTION" Rom type memory */
  .isr_vector :
  {
    . = ALIGN(4);
    PROVIDE(APPL_SECTION_START = .);
    KEEP(*(.isr_vector)) /* Startup code */
    . = ALIGN(4);
  } >APPL_SECTION

  /* The program code and other data into "APPL_SECTION" Rom type memory */
  .text :
  {
    . = ALIGN(4);
    *(.text)           /* .text sections (code) */
    *(.text*)          /* .text* sections (code) */
    *(.glue_7)         /* glue arm to thumb code */
    *(.glue_7t)        /* glue thumb to arm code */
    *(.eh_frame)

    KEEP (*(.init))
    KEEP (*(.fini))

    . = ALIGN(4);
    _etext = .;        /* define a global symbols at end of code */
  } >APPL_SECTION

  /* Constant data into "APPL_SECTION" Rom type memory */
  .rodata :
  {
    . = ALIGN(4);
    *(.rodata)         /* .rodata sections (constants, strings, etc.) */
    *(.rodata*)        /* .rodata* sections (constants, strings, etc.) */
    . = ALIGN(4);
  } >APPL_SECTION

  .ARM.extab   : {
    . = ALIGN(4);
    *(.ARM.extab* .gnu.linkonce.armextab.*)
    . = ALIGN(4);
  } >APPL_SECTION

  .ARM : {
    . = ALIGN(4);
    __exidx_start = .;
    *(.ARM.exidx*)
    __exidx_end = .;
    . = ALIGN(4);
  } >APPL_SECTION

  .preinit_array     :
  {
    . = ALIGN(4);
    PROVIDE_HIDDEN (__preinit_array_start = .);
    KEEP (*(.preinit_array*))
    PROVIDE_HIDDEN (__preinit_array_end = .);
    . = ALIGN(4);
  } >APPL_SECTION

  .init_array :
  {
    . = ALIGN(4);
    PROVIDE_HIDDEN (__init_array_start = .);
    KEEP (*(SORT(.init_array.*)))
    KEEP (*(.init_array*))
    PROVIDE_HIDDEN (__init_array_end = .);
    . = ALIGN(4);
  } >APPL_SECTION

  .fini_array :
  {
    . = ALIGN(4);
    PROVIDE_HIDDEN (__fini_array_start = .);
    KEEP (*(SORT(.fini_array.*)))
    KEEP (*(.fini_array*))
    PROVIDE_HIDDEN (__fini_array_end = .);
    . = ALIGN(4);
  } >APPL_SECTION


	/* Einstellungsdaten (in <STORING_SECTION> am Ende des Flashspeichers)*/
  .storingData :
  {
    . = ALIGN(4);
    PROVIDE(STORING_SECTION_START = .);
    *(storingData)
  } >STORING_SECTION

  /* Die Cy4Net-Addresse. Die Sektion ist nur 8 Byte groß und Teil der <STORING_SECTION>. Sie
     liegt ganz am Ende Flashspeichers.*/
  .cy4AdrSection :
  {
  	. = ALIGN(4);
    PROVIDE(CY4ADR_SECTION_START = .);
  } >CY4ADR_SECTION

  /* Used by the startup to initialize data */
  _sidata = LOADADDR(.data);

  /* Initialized data sections into "RAM" Ram type memory */
  .data :
  {
    . = ALIGN(4);
    _sdata = .;        /* create a global symbol at data start */
    *(.data)           /* .data sections */
    *(.data*)          /* .data* sections */
    *(.RamFunc)        /* .RamFunc sections */
    *(.RamFunc*)       /* .RamFunc* sections */

    . = ALIGN(4);
    _edata = .;        /* define a global symbol at data end */

  } >RAM AT> APPL_SECTION

  _siccmram = LOADADDR(.ccmram);

  /* CCM-RAM section
  *
  * IMPORTANT NOTE!
  * If initialized variables will be placed in this section,
  * the startup code needs to be modified to copy the init-values.
  */
  .ccmram :
  {
    . = ALIGN(4);
    _sccmram = .;       /* create a global symbol at ccmram start */
    *(.ccmram)
    *(.ccmram*)

    . = ALIGN(4);
    _eccmram = .;       /* create a global symbol at ccmram end */
  } >CCMRAM AT> APPL_SECTION

  /* Uninitialized data section into "RAM" Ram type memory */
  . = ALIGN(4);
  .bss :
  {
    /* This is used by the startup in order to initialize the .bss section */
    _sbss = .;         /* define a global symbol at bss start */
    __bss_start__ = _sbss;
    *(.bss)
    *(.bss*)
    *(COMMON)

    . = ALIGN(4);
    _ebss = .;         /* define a global symbol at bss end */
    __bss_end__ = _ebss;
  } >RAM

  /* User_heap_stack section, used to check that there is enough "RAM" Ram  type memory left */
  ._user_heap_stack :
  {
    . = ALIGN(8);
    PROVIDE ( end = . );
    PROVIDE ( _end = . );
    . = . + _Min_Heap_Size;
    . = . + _Min_Stack_Size;
    . = ALIGN(8);
  } >RAM

  /* Remove information from the compiler libraries */
  /DISCARD/ :
  {
    libc.a ( * )
    libm.a ( * )
    libgcc.a ( * )
  }

  .ARM.attributes 0 : { *(.ARM.attributes) }
}
