﻿//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  ARM Chipspezifische Basisfunktionalitäten                                           //
//  _________________________________                                                   //
//                                                                                      //
//  Copyright (c) 2022 Reimar Grasbon                                                   //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "stm32f3xx.h"

//////////////////////////////////////////////////////////////////////////////////////////
// Direktive um Daten im Flashspeicher zu platzieren  ////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
#define PROGMEM      __attribute__ ((section(".rodata")))
#define PSTR(s)      (__extension__({static const char __c[] PROGMEM = (s); &__c[0];}))
#define EVEN_ALIGNED __attribute__((aligned(2)))

//////////////////////////////////////////////////////////////////////////////////////////
//  Kritischer Abschnitt (Atomare Operation)  ////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
static __inline__ void EnableInterrupts(void)  { __enable_irq();  }
static __inline__ void DisableInterrupts(void) { __disable_irq(); }
static __inline__ uint8_t __iCli(void)
{
    DisableInterrupts();
    return 1;
}
static __inline__ void   __iSei(const uint8_t* v)
{
	(void)v;
    EnableInterrupts();
}
#define Atomic for (uint8_t _sreg_save_ __attribute__((__cleanup__(__iSei))) = 1,\
	           _it_ = __iCli(); _it_ != 0 ; _it_ = 0)


//////////////////////////////////////////////////////////////////////////////////////////
//  Freeze10us(): Aktive Warteschleife. Verzögert um ca. 10µs. Die Anzahl der Schlei-   //
//  ------------  fendurchläufe wird auf Grundlage der Prozessortaktrate in 			//
//	<HSE_VALUE> ermittelt.   															//
//                                                                                      //
//  Eingabe: -                                                                          //
//  Ausgabe: -                                                                          //
//////////////////////////////////////////////////////////////////////////////////////////
static __inline__ void Freeze10us(void)
{
	const uint32_t microseconds = 10;
	volatile uint32_t cycles = (microseconds*(HSE_VALUE/1000))/1000;
	__asm__ volatile (".syntax unified \n"
		        	  ".align 4        \n"
		        	  "1: SUBS %[cycles], %[cycles], #5 \n"
		        	  "   CMP %[cycles], #0 \n"
		        	  "   BPL 1b" : [cycles] "=l" (cycles)
		        	              : "0" (cycles)
		        	              : "cc");
}

//////////////////////////////////////////////////////////////////////////////////////////
//  Freeze100us(): Aktive Warteschleife. Verzögert um ca. 100µs. Die Anzahl der Schlei- //
//  -------------  fendurchläufe wird auf Grundlage der Prozessortaktrate in 			//
//	<HSE_VALUE> ermittelt.   															//
//                                                                                      //
//  Eingabe: -                                                                          //
//  Ausgabe: -                                                                          //
//////////////////////////////////////////////////////////////////////////////////////////
static __inline__ void Freeze100us(void)
{
	const uint32_t microseconds = 100;
	volatile uint32_t cycles = (microseconds*(HSE_VALUE/1000))/1000;
	__asm__ volatile (".syntax unified \n"
		        	  ".align 4        \n"
		        	  "1: SUBS %[cycles], %[cycles], #5 \n"
		        	  "   CMP %[cycles], #0 \n"
		        	  "   BPL 1b" : [cycles] "=l" (cycles)
		        	              : "0" (cycles)
		        	              : "cc");
}

//////////////////////////////////////////////////////////////////////////////////////////

